from typing import Optional

import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List
from discord.ext import commands
from discord.ext.commands import Context

from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_joined_server_repository_abc import UserJoinedServerRepositoryABC
from bot_data.abc.user_repository_abc import UserRepositoryABC
from modules.base.configuration.base_server_settings import BaseServerSettings
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class UserInfoCommandService(DiscordCommandABC):

    def __init__(
            self,
            config: ConfigurationABC,
            logger: LoggerABC,
            message_service: MessageServiceABC,
            bot: DiscordBotServiceABC,
            client_utils: ClientUtilsServiceABC,
            permissions: PermissionServiceABC,
            servers: ServerRepositoryABC,
            users: UserRepositoryABC,
            user_joined_servers: UserJoinedServerRepositoryABC
    ):
        DiscordCommandABC.__init__(self)

        self._config = config
        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._client_utils = client_utils
        self._permissions = permissions
        self._servers = servers
        self._users = users
        self._user_joined_servers = user_joined_servers

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.command(name='user-info')
    async def user_info(self, ctx: Context, member: Optional[discord.Member] = None, *, wait: int = None):
        #
        # Todo: Use native embeds!!!
        #
        self._logger.debug(__name__, f'Received command user-info {ctx}:{member},{wait}')
        self._client_utils.received_command(ctx.guild.id)
        settings: BaseServerSettings = self._config.get_configuration(f'BaseServerSettings_{ctx.guild.id}')

        if not self._permissions.is_member_moderator(ctx.author):
            await self._message_service.send_ctx_msg(ctx, settings.no_permission_message)
            self._logger.trace(__name__, f'Finished purge command')
            return

        if member is None or not isinstance(member, discord.Member):
            member = ctx.author

        server = self._servers.find_server_by_discord_id(ctx.guild.id)
        user = self._users.find_user_by_discord_id_and_server_id(member.id, server.server_id)
        joins = self._user_joined_servers.get_user_joined_servers_by_user_id(user.user_id)

        embed_description = EmbedDescription(
            settings.user_info_command_message.embed_description.title.format(member.name),
            settings.user_info_command_message.embed_description.description.format(member.name),
            settings.user_info_command_message.embed_description.url,
            settings.user_info_command_message.embed_description.color,
            List(EmbedDescriptionField),
            settings.user_info_command_message.embed_description.footer
        )

        for i in range(len(settings.user_info_command_message.embed_description.fields)):
            settings_field = settings.user_info_command_message.embed_description.fields[i]
            field = EmbedDescriptionField(settings_field.name, settings_field.value, settings_field.inline)

            if settings_field.value == '$id':
                field.value = settings_field.value.replace('$id', str(member.id))

            elif settings_field.value == '$name':
                field.value = settings_field.value.replace('$name', member.name)

            elif settings_field.value == '$discord_join':
                field.value = settings_field.value.replace('$discord_join', str(member.created_at))

            elif settings_field.value == '$last_join':
                field.value = settings_field.value.replace('$last_join', str(member.joined_at))

            elif settings_field.value == '$xp':
                field.value = settings_field.value.replace('$xp', str(user.xp))

            elif settings_field.value == '$roles':
                roles = ''
                for role in member.roles:
                    roles += f'{role.name}\n'
                field.value = settings_field.value.replace('$roles', roles)

            elif settings_field.value == '$joins':
                joins_string = ''
                for join in joins:
                    joins_string += f'{join.joined_on}\n'
                field.value = settings_field.value.replace('$joins', joins_string)

            elif settings_field.value == '$leavings':
                leavings_string = ''
                for join in joins:
                    if join.leaved_on is None:
                        if leavings_string == '':
                            leavings_string = '/'
                        continue
                    leavings_string += f'{join.leaved_on}\n'
                field.value = settings_field.value.replace('$leavings', leavings_string)

            elif settings_field.value == '$warnings':
                field.value = 'Not Implemented yet'

            embed_description.fields.append(field)

        await self._message_service.send_ctx_msg(
            ctx,
            EmbedService.get_embed(embed_description),
            wait_before_delete=wait
        )
        self._logger.trace(__name__, f'Finished user-info command')
